Package: trackViewer
Type: Package
Title: A R/Bioconductor package with web interface for drawing elegant
        interactive tracks or lollipop plot to facilitate integrated
        analysis of multi-omics data
Version: 1.49.0
Authors@R: c(person(given="Jianhong", family="Ou", email="jou@morgridge.org", 
        role=c("aut", "cre"), comment=c(ORCID="0000-0002-8652-2488")), 
        person(given="Julie Lihua", family="Zhu", 
        role="aut", email="Julie.Zhu@umassmed.edu"))
Maintainer: Jianhong Ou <jou@morgridge.org>
Description: Visualize mapped reads along with annotation as track
        layers for NGS dataset such as ChIP-seq, RNA-seq, miRNA-seq,
        DNA-seq, SNPs and methylation data.
License: GPL (>= 2)
Depends: R (>= 3.5.0), grDevices, methods, GenomicRanges, grid
Imports: Seqinfo, GenomeInfoDb, GenomicAlignments, GenomicFeatures,
        Gviz, Rsamtools, S4Vectors, rtracklayer, BiocGenerics, scales,
        tools, IRanges, AnnotationDbi, grImport, htmlwidgets,
        InteractionSet, utils, rhdf5, strawr, txdbmaker
Suggests: biomaRt, TxDb.Hsapiens.UCSC.hg19.knownGene, RUnit,
        org.Hs.eg.db, BiocStyle, knitr, VariantAnnotation, httr,
        htmltools, rmarkdown, motifStack
biocViews: Visualization
VignetteBuilder: knitr
RoxygenNote: 7.3.3
Encoding: UTF-8
Config/pak/sysreqs: cmake make libbz2-dev libicu-dev libjpeg-dev
        liblzma-dev libpng-dev libuv1-dev libxml2-dev libssl-dev
        xz-utils zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-04-28 12:38:35 UTC
RemoteUrl: https://github.com/bioc/trackViewer
RemoteRef: HEAD
RemoteSha: ea7da89d27c3c123f98fcd043e958c269a1a997b
NeedsCompilation: no
Packaged: 2026-05-30 09:52:54 UTC; root
Author: Jianhong Ou [aut, cre] (ORCID: <https://orcid.org/0000-0002-8652-2488>),
  Julie Lihua Zhu [aut]
Built: R 4.6.0; ; 2026-05-30 15:16:04 UTC; unix
