# Generated by roxygen2: do not edit by hand

export(ARA)
export(GIoperator)
export(GRoperator)
export(addArrowMark)
export(addGuideLine)
export(addInteractionAnnotation)
export(browseTracks)
export(browseTracksOutput)
export(coverageGR)
export(dandelion.plot)
export(geneModelFromTxdb)
export(geneTrack)
export(getCurTrackViewport)
export(getGeneIDsFromTxDb)
export(getLocation)
export(gi2track)
export(gieStain)
export(ideogramPlot)
export(importBam)
export(importData)
export(importGInteractions)
export(importScore)
export(listChromosomes)
export(listNormalizations)
export(listResolutions)
export(loadIdeogram)
export(lolliplot)
export(optimizeStyle)
export(parse2GRanges)
export(parseWIG)
export(plotGRanges)
export(renderbrowseTracks)
export(trackList)
export(trackViewerStyle)
export(viewGene)
export(viewTracks)
exportClasses(pos)
exportClasses(track)
exportClasses(trackList)
exportClasses(trackStyle)
exportClasses(trackViewerStyle)
exportClasses(xscale)
exportClasses(yaxisStyle)
exportMethods("$")
exportMethods("$<-")
exportMethods("seqlevelsStyle<-")
exportMethods(reduce)
exportMethods(seqlevels)
exportMethods(seqlevelsStyle)
exportMethods(setTrackStyleParam)
exportMethods(setTrackViewerStyleParam)
exportMethods(setTrackXscaleParam)
exportMethods(setTrackYaxisParam)
exportMethods(show)
import(GenomicAlignments)
import(GenomicRanges)
import(IRanges)
import(Rsamtools)
import(S4Vectors)
import(grid)
import(htmlwidgets)
import(methods)
importClassesFrom(grImport,Picture)
importFrom(AnnotationDbi,columns)
importFrom(AnnotationDbi,mapIds)
importFrom(AnnotationDbi,mget)
importFrom(AnnotationDbi,select)
importFrom(BiocGenerics,basename)
importFrom(BiocGenerics,strand)
importFrom(GenomeInfoDb,"seqlevelsStyle<-")
importFrom(GenomeInfoDb,seqlevelsStyle)
importFrom(GenomicFeatures,genes)
importFrom(GenomicFeatures,transcripts)
importFrom(Gviz,GeneRegionTrack)
importFrom(IRanges,Views)
importFrom(IRanges,viewMaxs)
importFrom(InteractionSet,GInteractions)
importFrom(InteractionSet,`regions<-`)
importFrom(InteractionSet,anchorIds)
importFrom(InteractionSet,regions)
importFrom(S4Vectors,first)
importFrom(S4Vectors,second)
importFrom(Seqinfo,"genome<-")
importFrom(Seqinfo,"seqinfo<-")
importFrom(Seqinfo,"seqlevels<-")
importFrom(Seqinfo,Seqinfo)
importFrom(Seqinfo,genome)
importFrom(Seqinfo,isCircular)
importFrom(Seqinfo,seqinfo)
importFrom(Seqinfo,seqlengths)
importFrom(Seqinfo,seqlevels)
importFrom(Seqinfo,seqnames)
importFrom(grDevices,as.raster)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,palette)
importFrom(grDevices,rgb)
importFrom(grImport,grid.picture)
importFrom(grImport,readPicture)
importFrom(rhdf5,H5Fopen)
importFrom(rhdf5,H5close)
importFrom(rhdf5,h5closeAll)
importFrom(rhdf5,h5ls)
importFrom(rhdf5,h5read)
importFrom(rtracklayer,GRangesForUCSCGenome)
importFrom(rtracklayer,browserSession)
importFrom(rtracklayer,getTable)
importFrom(rtracklayer,import)
importFrom(rtracklayer,ucscTableQuery)
importFrom(scales,rescale)
importFrom(stats,ksmooth)
importFrom(strawr,readHicBpResolutions)
importFrom(strawr,readHicChroms)
importFrom(strawr,readHicNormTypes)
importFrom(strawr,straw)
importFrom(tools,file_ext)
importFrom(txdbmaker,makeTxDb)
importFrom(utils,combn)
importFrom(utils,installed.packages)
importMethodsFrom(GenomicRanges,range)
importMethodsFrom(GenomicRanges,reduce)
